/*
**　高精度ＡＤコンバータ　ＡＤ７７０５
**　１６ビットΣΔ型、リファレンス電圧２．５Ｖ　差動入力
**  積分型電流計
*/

//　液晶モジュール用ヘッダファイル
#include <LiquidCrystal.h>
//　EEPROM用ヘッダファイル
#include <EEPROM.h>

#define COUNT2mV 237.2    // ＡＤ７７０５出力値から電圧への変換係数

//　液晶モジュールのピン設定
LiquidCrystal lcd(8, 9, 4, 5, 6, 7);

// ＡＤ７７０５とのインターフェイス（入出力はArdionoのポートの方向）
int DRDY = 2;  // データレディ（入力）
int CS = 3;    //　チップセレクト（出力）
int DIN = 11;  //　データ（出力）
int DOUT = 12;  //　データ（入力）
int SCLK = 13;   //　シリアルクロック（出力）

// 積分器とのインターフェイス
int izero = 14;
int ipass = 15;
int isample = 16;

//　ａｄ７７０５への１バイト送信（ＳＣＬＫとＤＩＮのみ駆動）
void send_AD7705( unsigned char d ) {
  unsigned char i;
  for( i = 0; i < 8; i ++ ) {  //　８ビット分繰り返し
    digitalWrite( SCLK, LOW ); //　クロック線をＬＯＷレベルにする
    if( d & 0x80 ) digitalWrite( DIN, HIGH ); //　データビットを準備
    else           digitalWrite( DIN, LOW );
    digitalWrite( SCLK, HIGH );  //　クロック線をＨＩＧＨレベルに戻す
    d *= 2;  //　左シフト
  }
}

//　ＡＤ７７０５から１バイト受信（ＳＣＬＫのみ駆動）
unsigned char recv_AD7705() {
  unsigned char i, d;
  d = 0;
  for( i = 0; i < 8; i ++ ) {
    digitalWrite( SCLK, LOW ); //　クロック線をＬＯＷレベルにする
    d *= 2; //左シフト  
    if( HIGH == digitalRead( DOUT ) ) d += 1;  // HIGHならビットを立てる
    digitalWrite( SCLK, HIGH );  //　クロック線をＨＩＧＨレベルに戻す
  }
  return d;
}

//　ＡＤ７７０５の初期化
void setup_AD7705() {
//　１．ポートの向きを設定
  pinMode( DRDY, INPUT );
  pinMode( CS, OUTPUT );
  pinMode( DIN, OUTPUT );
  pinMode( DOUT, INPUT );
  pinMode( SCLK, OUTPUT );
//　２．出力ポートはＨレベルにしておく
  digitalWrite( CS, HIGH );
  digitalWrite( DIN, HIGH );
  digitalWrite( SCLK, HIGH );
// ＡＤ７７０５シリアルポート初期化
  delay( 10 );    // １０ｍＳ待ち
  digitalWrite( CS, LOW );  //　アクセス開始
  send_AD7705( 0xff );    //　最低３２ビットのＨレベルを送信することで
  send_AD7705( 0xff );    //　ＡＤ７７０５のシリアルポートをリセットできる
  send_AD7705( 0xff );
  send_AD7705( 0xff );
  send_AD7705( 0xff );
  digitalWrite( CS, HIGH );  //　リセット終了
  delay( 1 );
// 初期化:クロック設定
  digitalWrite( CS, LOW );  //　アクセス開始
  send_AD7705( 0x20 );    //　クロックレジスタを選択
  digitalWrite( CS, HIGH ); 
  delayMicroseconds( 10 );
  digitalWrite( CS, LOW );  //　アクセス開始
  send_AD7705( 0x08 );    //　クロック、出力更新レートを選択
  // 電源周波数が６０Ｈｚの地域では０ｘ０８、５０Ｈｚの地域では０ｘ０９
  digitalWrite( CS, HIGH );
  delayMicroseconds( 10 );
// 初期化:セットアップレジスタ設定による自動校正
  digitalWrite( CS, LOW );  //　アクセス開始
  send_AD7705( 0x10 );    //　セットアップレジスタを選択
  digitalWrite( CS, HIGH );
  delayMicroseconds( 10 );
  digitalWrite( CS, LOW );  //　アクセス開始
  send_AD7705( 0x40 );    //　バイポーラ動作、セルフキャリブレーション実行
  digitalWrite( CS, HIGH ); 
  delay( 1 );
  while( LOW == digitalRead( DRDY ) ) {  // 校正終了待ち
    delay( 1 );
  }
// 初期化:セットアップレジスタ設定によるゲイン設定
  digitalWrite( CS, LOW );  //　アクセス開始
  send_AD7705( 0x10 );    //　セットアップレジスタを選択
  digitalWrite( CS, HIGH );
  delayMicroseconds( 10 );
  digitalWrite( CS, LOW );  //　アクセス開始
  send_AD7705( 0x08 );    //　バイポーラ動作、ゲイン１倍に切り替え
  digitalWrite( CS, HIGH ); 
}

// ＡＤ７７０５からのデータ読み出し（変換が終わるまで待ってデータを読み出す）
unsigned int conv_AD7705() {
  unsigned int d;
  unsigned char h, l;

  while( LOW == digitalRead( DRDY ) ) {  // 変換終了待ち
    delay( 1 );
  }
  digitalWrite( CS, LOW );  //　アクセス開始
  send_AD7705( 0x38 );    //　データレジスタ読み出しを選択
  digitalWrite( CS, HIGH );
  delayMicroseconds( 10 );
  digitalWrite( CS, LOW );  //　アクセス開始
  h = recv_AD7705();        //　上位バイト
  l = recv_AD7705();        //　下位バイト
  digitalWrite( CS, HIGH );
  d = ( h << 8 ) + l;
  return d;
}

//　ゼロ点を記憶するエリア
unsigned long int zero;

//　初期化のために最初に１回だけ呼ばれる関数
void setup() {
  int i;

  lcd.begin(16, 2);        //　液晶モジュールの初期化
  Serial.begin( 9600 );   //　シリアルポートの初期化
  pinMode( izero, OUTPUT );  // 積分器関連初期化
  pinMode( ipass, OUTPUT );
  pinMode( isample, OUTPUT );
  digitalWrite( izero, HIGH );
  digitalWrite( ipass, HIGH );
  digitalWrite( isample, LOW );
  setup_AD7705();          //　ＡＤ７７０５の初期化
  delay( 1000 );           //　安定待ち

  digitalWrite( CS, LOW );  //　アクセス開始
  send_AD7705( 0x02 );    //　コミュニケーションレジスタでチャンネル０、入力なしを選択
  digitalWrite( CS, HIGH ); 
  delayMicroseconds( 10 );
  digitalWrite( CS, LOW );  //　アクセス開始
  send_AD7705( 0x02 );    //　コミュニケーションレジスタでチャンネル０、入力なしを選択
  digitalWrite( CS, HIGH ); 
  delay( 10 );

  zero = 0;
  for( i = 0; i < 10; i ++ ) zero += conv_AD7705();  // １０回分の総和を保管
  
  digitalWrite( CS, LOW );  //　アクセス開始
  send_AD7705( 0x00 );    //　コミュニケーションレジスタでチャンネル０選択
  digitalWrite( CS, HIGH ); 
  delayMicroseconds( 10 );
  digitalWrite( CS, LOW );  //　アクセス開始
  send_AD7705( 0x00 );    //　コミュニケーションレジスタでチャンネル０、入力なしを選択
  digitalWrite( CS, HIGH ); 
  delay( 10 );
}

//　動作時に繰り返し呼ばれる関数
void loop() {
  long int d;
  int i;
  float v;


  digitalWrite( izero, HIGH );   // auto zero
  digitalWrite( ipass, HIGH );
  delay( 100 );
  digitalWrite( izero, LOW );    //sample start
  digitalWrite( isample, HIGH );
  digitalWrite( ipass, LOW );
  delay( 1000 );
  digitalWrite( ipass, HIGH );   // sample stop
  digitalWrite( isample, LOW );  // and hold
  d = - zero;
  for( i = 0; i < 10; i ++ ) d += conv_AD7705();
  v = ( float )d / COUNT2mV;
  lcd.setCursor( 0, 0 );
  lcd.print( v );
  lcd.print( " nA  " );
  Serial.println( v );
  digitalWrite( izero, HIGH );  // discharge
}

